@@ -31,6 +31,15 @@
    /* We are using this library */
 #  define AKONADI_EXPORT KDE_IMPORT
 # endif
+
+# ifndef AKONADI_EXPORT_DEPRECATED
+#   if defined(MAKE_AKONADI_KDE_LIB) || defined AKONADI_IGNORE_DEPRECATED_WARNINGS
+#     define AKONADI_EXPORT_DEPRECATED AKONADI_EXPORT
+#   else
+#     define AKONADI_EXPORT_DEPRECATED KDE_DEPRECATED AKONADI_EXPORT
+#   endif
+# endif
+
 #endif
 
 #endif
