@@ -28,6 +28,8 @@
 #include <akonadi/collectionfetchscope.h>
 #include <akonadi/session.h>
 
+#include "akonadiprivate_export.h"
+
 #include <qobject.h>
 #include <QQueue>
 #include <QVariant>
@@ -41,7 +43,7 @@ namespace Akonadi {
   @internal
   QObject part of EntityCache.
 */
-class EntityCacheBase : public QObject
+class AKONADI_TESTS_EXPORT EntityCacheBase : public QObject
 {
   Q_OBJECT
   public:
@@ -153,7 +155,7 @@ class EntityCache : public EntityCacheBase
       FetchJob* job = createFetchJob( id );
       job->setFetchScope( scope );
       job->setProperty( "EntityCacheNode", QVariant::fromValue<typename T::Id>( id ) );
-      connect( job, SIGNAL(result(KJob*)), SLOT(fetchResult(KJob*)) );
+      connect( job, SIGNAL( result( KJob* ) ), SLOT( fetchResult( KJob* ) ) );
       mCache.enqueue( node );
     }
 
