@@ -53,6 +53,11 @@ class KCAL_EXPORT Journal : public Incidence
     */
     typedef boost::shared_ptr<Journal> Ptr;
 
+    /**
+      A shared pointer to a non-mutable Journal object.
+    */
+    typedef boost::shared_ptr<const Journal> ConstPtr;
+
     /**
       Constructs an empty journal.
     */
@@ -69,6 +74,12 @@ class KCAL_EXPORT Journal : public Incidence
     */
     QByteArray type() const;
 
+    /**
+      @copydoc
+      IncidenceBase::typeStr()
+    */
+    //KDE5: QString typeStr() const;
+
     /**
       Returns an exact copy of this journal. The returned object is owned
       by the caller.
