@@ -112,13 +112,13 @@ void GetMetaDataJob::doStart()
   }
 
   d->tags << d->sessionInternal()->sendCommand( command, parameters );
-  kDebug() << "SENT: " << command << " " << parameters;
+//  kDebug() << "SENT: " << command << " " << parameters;
 }
 
 void GetMetaDataJob::handleResponse( const Message &response )
 {
   Q_D(GetMetaDataJob);
-  kDebug() << "GOT: " << response.toString();
+//  kDebug() << "GOT: " << response.toString();
 
   //TODO: handle NO error messages having [METADATA MAXSIZE NNN], [METADATA TOOMANY], [METADATA NOPRIVATE] (see rfc5464)
   // or [ANNOTATEMORE TOOBIG], [ANNOTATEMORE TOOMANY] respectively
