@@ -167,18 +167,69 @@ void LoginJob::setPassword( const QString &password )
 void LoginJob::doStart()
 {
   Q_D(LoginJob);
-  if (d->encryptionMode == SslV2 || d->encryptionMode == SslV3 || d->encryptionMode == SslV3_1 || d->encryptionMode == AnySslVersion) {
+
+  // Don't authenticate on a session in the authenticated state
+  if ( session()->state() == Session::Authenticated || session()->state() == Session::Selected ) {
+    setError( UserDefinedError );
+    setErrorText( i18n("IMAP session in the wrong state for authentication") );
+    emitResult();
+    return;
+  }
+
+  // Trigger encryption negociation only if needed
+  EncryptionMode encryptionMode = d->encryptionMode;
+
+  switch ( d->sessionInternal()->negotiatedEncryption() ) {
+  case KTcpSocket::UnknownSslVersion:
+    break; // Do nothing the encryption mode still needs to be negociated
+
+  // For the other cases, pretend we're going unencrypted as that's the
+  // encryption mode already set on the session
+  // (so for instance we won't issue another STARTTLS for nothing if that's
+  // not needed)
+  case KTcpSocket::SslV2:
+    if ( encryptionMode==SslV2 ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  case KTcpSocket::SslV3:
+    if ( encryptionMode==SslV3 ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  case KTcpSocket::TlsV1:
+    if ( encryptionMode==TlsV1 ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  case KTcpSocket::AnySslVersion:
+    if ( encryptionMode==AnySslVersion ) {
+      encryptionMode = Unencrypted;
+    }
+    break;
+  }
+
+  if (encryptionMode == SslV2
+   || encryptionMode == SslV3
+   || encryptionMode == SslV3_1
+   || encryptionMode == AnySslVersion) {
     KTcpSocket::SslVersion version = KTcpSocket::SslV2;
-    if (d->encryptionMode == SslV3)
+    if (encryptionMode == SslV3)
       version = KTcpSocket::SslV3;
-    if (d->encryptionMode == SslV3_1)
+    if (encryptionMode == SslV3_1)
       version = KTcpSocket::SslV3_1;
-    if (d->encryptionMode == AnySslVersion)
+    if (encryptionMode == AnySslVersion)
       version = KTcpSocket::AnySslVersion;
     d->sessionInternal()->startSsl(version);
-  } else  if (d->encryptionMode == Unencrypted ) {
+
+  } else if (encryptionMode == TlsV1) {
+    d->authState = LoginJobPrivate::StartTls;
+    d->tags << d->sessionInternal()->sendCommand( "STARTTLS" );
+
+  } else  if (encryptionMode == Unencrypted ) {
     if (d->authMode.isEmpty()) {
-      d->tag = d->sessionInternal()->sendCommand( "LOGIN",
+      d->authState = LoginJobPrivate::Login;
+      d->tags << d->sessionInternal()->sendCommand( "LOGIN",
                                                   quoteIMAP( d->userName ).toUtf8()
                                                  +' '
                                                  +quoteIMAP(d->password ).toUtf8() );
@@ -187,9 +238,6 @@ void LoginJob::doStart()
         emitResult();
       }
     }
-  } else if (d->encryptionMode == TlsV1) {
-    d->authState = LoginJobPrivate::StartTls;
-    d->tag = d->sessionInternal()->sendCommand( "STARTTLS" );
   }
 }
 
@@ -205,8 +253,21 @@ void LoginJob::handleResponse( const Message &response )
     commandName = i18n("StartTls");
   }
 
-  if ( !response.content.isEmpty()
-       && response.content.first().toString() == d->tag ) {
+  if ( d->authMode == QLatin1String( "PLAIN" ) && !response.content.isEmpty() && response.content.first().toString()=="+" ) {
+    if ( response.content.size()>1 && response.content.at( 1 ).toString()=="OK" ) {
+      return;
+    }
+
+    QByteArray challengeResponse;
+    challengeResponse+= '\0';
+    challengeResponse+= d->userName.toUtf8();
+    challengeResponse+= '\0';
+    challengeResponse+= d->password.toUtf8();
+    challengeResponse = challengeResponse.toBase64();
+    d->sessionInternal()->sendData( challengeResponse );
+
+  } else if ( !response.content.isEmpty()
+       && d->tags.contains( response.content.first().toString() ) ) {
     if ( response.content.size() < 2 ) {
       setErrorText( i18n("%1 failed, malformed reply from the server.", commandName) );
       emitResult();
@@ -233,7 +294,7 @@ void LoginJob::handleResponse( const Message &response )
             emitResult();
           } else {
             d->authState = LoginJobPrivate::Login;
-            d->tag = d->sessionInternal()->sendCommand( "LOGIN",
+            d->tags << d->sessionInternal()->sendCommand( "LOGIN",
                                                         quoteIMAP( d->userName ).toUtf8()
                                                        +' '
                                                        +quoteIMAP( d->password ).toUtf8() );
@@ -241,8 +302,8 @@ void LoginJob::handleResponse( const Message &response )
         }
 
         //find the selected SASL authentication method
-        Q_FOREACH(QString capability, d->capabilities) {
-          if (capability.startsWith("AUTH=")) {
+        Q_FOREACH(const QString &capability, d->capabilities) {
+          if (capability.startsWith(QLatin1String("AUTH="))) {
             QString authType = capability.mid(5);
             if (authType == d->authMode) {
                 if (!d->startAuthentication()) {
@@ -259,7 +320,7 @@ void LoginJob::handleResponse( const Message &response )
     }
   } else if ( response.content.size() >= 2 ) {
     if ( d->authState == LoginJobPrivate::Authenticate ) {
-      if (!d->answerChallenge(response.content[1].toString())) {
+      if (!d->answerChallenge(QByteArray::fromBase64(response.content[1].toString()))) {
         emitResult(); //error, we're done
       }
     } else if ( response.content[1].toString()=="CAPABILITY" ) {
@@ -330,7 +391,11 @@ bool LoginJobPrivate::startAuthentication()
   QByteArray tmp = QByteArray::fromRawData( out, outlen );
   QByteArray challenge = tmp.toBase64();
 
-  tag = sessionInternal()->sendCommand( "AUTHENTICATE", authMode.toLatin1() + ' ' + challenge );
+  if ( challenge.isEmpty() ) {
+    tags << sessionInternal()->sendCommand( "AUTHENTICATE", authMode.toLatin1() );
+  } else {
+    tags << sessionInternal()->sendCommand( "AUTHENTICATE", authMode.toLatin1() + ' ' + challenge );
+  }
 
   return true;
 }
@@ -376,7 +441,7 @@ void LoginJobPrivate::sslResponse(bool response)
 {
   if (response) {
     authState = LoginJobPrivate::Capability;
-    tag = sessionInternal()->sendCommand( "CAPABILITY" );
+    tags << sessionInternal()->sendCommand( "CAPABILITY" );
   } else {
     q->setError( LoginJob::UserDefinedError );
     q->setErrorText( i18n("Login failed, TLS negotiation failed." ));
