@@ -1,7 +1,7 @@
 /*  -*- c++ -*-
     kmime_header_parsing.cpp
 
-    KMime, the KDE internet mail/usenet news message library.
+    KMime, the KDE Internet mail/usenet news message library.
     Copyright (c) 2001-2002 Marc Mutz <mutz@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -23,7 +23,10 @@
 #include "kmime_header_parsing.h"
 
 #include "kmime_codecs.h"
+#include "kmime_headerfactory_p.h"
+#include "kmime_headers.h"
 #include "kmime_util.h"
+#include "kmime_util_p.h"
 #include "kmime_dateformatter.h"
 #include "kmime_warning.h"
 
@@ -78,7 +81,10 @@ static QString addr_spec_as_string( const AddrSpec & as, bool pretty )
   }
   const QString dom = pretty ? QUrl_fromAce_wrapper( as.domain ) : as.domain ;
   if ( needsQuotes ) {
-    return '"' + result + "\"@" + dom;
+    result = '"' + result + "\"";
+  }
+  if( dom.isEmpty() ) {
+    return result;
   } else {
     return result + '@' + dom;
   }
@@ -155,11 +161,20 @@ bool Mailbox::hasName() const
 }
 
 QString Mailbox::prettyAddress() const
+{
+  return prettyAddress( QuoteNever );
+}
+
+QString Mailbox::prettyAddress( Quoting quoting ) const
 {
   if ( !hasName() ) {
     return address();
   }
   QString s = name();
+  if ( quoting != QuoteNever ) {
+    addQuotes( s, quoting == QuoteAlways /*bool force*/ );
+  }
+
   if ( hasAddress() ) {
     s += QLatin1String(" <") + address() + QLatin1Char('>');
   }
@@ -217,8 +232,8 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   char ch = *scursor++;
 
   if ( ch != '?' ) {
-    // kDebug(5320) << "first";
-    KMIME_WARN_PREMATURE_END_OF( EncodedWord );
+    // kDebug() << "first";
+    //KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
 
@@ -239,7 +254,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
 
   // not found? can't be an encoded-word!
   if ( scursor == send || *scursor != '?' ) {
-    // kDebug(5320) << "second";
+    // kDebug() << "second";
     KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
@@ -270,7 +285,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
 
   // not found? Can't be an encoded-word!
   if ( scursor == send || *scursor != '?' ) {
-    // kDebug(5320) << "third";
+    // kDebug() << "third";
     KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
@@ -278,7 +293,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   // extract the encoding information:
   QByteArray maybeEncoding( encodingStart, scursor - encodingStart );
 
-  // kDebug(5320) << "parseEncodedWord: found charset == \"" << maybeCharset
+  // kDebug() << "parseEncodedWord: found charset == \"" << maybeCharset
   //         << "\"; language == \"" << maybeLanguage
   //         << "\"; encoding == \"" << maybeEncoding << "\"";
 
@@ -291,28 +306,31 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   scursor++;
   const char * encodedTextStart = scursor;
 
-  // find next '?' (ending the encoded-text):
+  // find the '?=' sequence (ending the encoded-text):
   for ( ; scursor != send ; scursor++ ) {
     if ( *scursor == '?' ) {
-      break;
+      if ( scursor + 1 != send ) {
+        if ( *( scursor + 1 ) != '=' ) { // We expect a '=' after the '?', but we got something else; ignore
+          KMIME_WARN << "Stray '?' in q-encoded word, ignoring this.";
+          continue;
+        }
+        else { // yep, found a '?=' sequence
+          scursor += 2;
+          break;
+        }
+      }
+      else { // The '?' is the last char, but we need a '=' after it!
+        KMIME_WARN_PREMATURE_END_OF( EncodedWord );
+        return false;
+      }
     }
   }
 
-  // not found? Can't be an encoded-word!
-  // ### maybe evaluate it nonetheless if the rest is OK?
-  if ( scursor == send || *scursor != '?' ) {
-    // kDebug(5320) << "fourth";
+  if ( *( scursor - 2 ) != '?' || *( scursor - 1 ) != '=' ||
+       scursor < encodedTextStart + 2 ) {
     KMIME_WARN_PREMATURE_END_OF( EncodedWord );
     return false;
   }
-  scursor++;
-  // check for trailing '=':
-  if ( scursor == send || *scursor != '=' ) {
-    // kDebug(5320) << "fifth";
-    KMIME_WARN_PREMATURE_END_OF( EncodedWord );
-    return false;
-  }
-  scursor++;
 
   // set end sentinel for encoded-text:
   const char * const encodedTextEnd = scursor - 2;
@@ -355,7 +373,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
     return false;
   };
 
-  // kDebug(5320) << "mimeName(): \"" << textCodec->name() << "\"";
+  // kDebug() << "mimeName(): \"" << textCodec->name() << "\"";
 
   // allocate a temporary buffer to store the 8bit text:
   int encodedTextLength = encodedTextEnd - encodedTextStart;
@@ -376,7 +394,7 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
 
   result = textCodec->toUnicode( buffer.data(), bbegin - buffer.data() );
 
-  // kDebug(5320) << "result now: \"" << result << "\"";
+  // kDebug() << "result now: \"" << result << "\"";
   // cleanup:
   delete dec;
   language = maybeLanguage;
@@ -433,6 +451,8 @@ bool parseAtom( const char * &scursor, const char * const send,
   return success;
 }
 
+// FIXME: Remove this and the other parseToken() method. add a new one where "result" is a
+//        QByteArray.
 bool parseToken( const char * &scursor, const char * const send,
                  QString &result, bool allow8Bit )
 {
@@ -485,6 +505,8 @@ bool parseToken( const char * &scursor, const char * const send,
 //
 // - doesn't handle quoted CRLF
 
+// FIXME: Why is result a QString? This should be a QByteArray, since at this level, we don't
+//        know about encodings yet!
 bool parseGenericQuotedString( const char* &scursor, const char * const send,
                                QString &result, bool isCRLF,
                                const char openChar, const char closeChar )
@@ -572,6 +594,29 @@ bool parseGenericQuotedString( const char* &scursor, const char * const send,
         scursor--;
       }
       break;
+    case '=':
+    {
+      // ### Work around broken clients that send encoded words in quoted-strings
+      //     For example, older KMail versions.
+      if( scursor == send )
+        break;
+      
+      const char *oldscursor = scursor;
+      QString tmp;
+      QByteArray lang, charset;
+      if( *scursor++ == '?' ) {
+        --scursor;
+        if( parseEncodedWord( scursor, send, tmp, lang, charset ) ) {
+          result += tmp;
+          break;
+        } else {
+          scursor = oldscursor;
+        }
+      } else {
+        scursor = oldscursor;
+      }
+      // fall through
+    }
     default:
       KMIME_WARN_IF_8BIT( ch );
       result += QChar( ch );
@@ -1102,6 +1147,15 @@ bool parseAngleAddr( const char* &scursor, const char * const send,
 
 }
 
+static QString stripQuotes( const QString &input )
+{
+  if ( input.startsWith( '"' ) && input.endsWith( '"' ) ) {
+    QString stripped( input.mid( 1, input.size() - 2 ) );
+    return stripped;
+  }
+  else return input;
+}
+
 bool parseMailbox( const char* &scursor, const char * const send,
                    Mailbox &result, bool isCRLF )
 {
@@ -1125,7 +1179,7 @@ bool parseMailbox( const char* &scursor, const char * const send,
         return false;
       }
     }
-    result.setNameFrom7Bit( maybeDisplayName.toLatin1() );
+    result.setName( stripQuotes( maybeDisplayName ) );
     return true;
   }
   scursor = oldscursor;
@@ -1159,7 +1213,7 @@ bool parseMailbox( const char* &scursor, const char * const send,
     }
   }
 
-  result.setName( maybeDisplayName );
+  result.setName( stripQuotes( maybeDisplayName ) );
   result.setAddress( maybeAddrSpec );
   return true;
 }
@@ -1313,6 +1367,9 @@ static QString asterisk = QString::fromLatin1( "*0*", 1 );
 static QString asteriskZero = QString::fromLatin1( "*0*", 2 );
 //static QString asteriskZeroAsterisk = QString::fromLatin1( "*0*", 3 );
 
+// FIXME: Get rid of the very ugly "QStringOrQPair" thing. At this level, we are supposed to work
+//        on byte arrays, not strings! The result parameter should be a simple
+//        QPair<QByteArray,QByteArray>, which is the attribute name and the value.
 bool parseParameter( const char* &scursor, const char * const send,
                      QPair<QString,QStringOrQPair> &result, bool isCRLF )
 {
@@ -1334,6 +1391,7 @@ bool parseParameter( const char* &scursor, const char * const send,
   //
   // parse the parameter name:
   //
+  // FIXME: maybeAttribute should be a QByteArray
   QString maybeAttribute;
   if ( !parseToken( scursor, send, maybeAttribute, false /* no 8bit */ ) ) {
     return false;
@@ -1394,6 +1452,8 @@ bool parseParameter( const char* &scursor, const char * const send,
   return true;
 }
 
+// FIXME: Get rid of QStringOrQPair: Use a simply QMap<QByteArray, QByteArray> for "result"
+//        instead!
 bool parseRawParameterList( const char* &scursor, const char * const send,
                             QMap<QString,QStringOrQPair> &result,
                             bool isCRLF )
@@ -1462,7 +1522,7 @@ bool parseRawParameterList( const char* &scursor, const char * const send,
 static void decodeRFC2231Value( Codec* &rfc2231Codec,
                                 QTextCodec* &textcodec,
                                 bool isContinuation, QString &value,
-                                QPair<const char*,int> &source )
+                                QPair<const char*,int> &source, QByteArray& charset )
 {
   //
   // parse the raw value into (charset,language,text):
@@ -1491,7 +1551,7 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
       return;
     }
 
-    QByteArray charset( decBegin, decCursor - decBegin );
+    charset = QByteArray( decBegin, decCursor - decBegin );
 
     const char * oldDecCursor = ++decCursor;
     // find the second single quote (we ignore the language tag):
@@ -1555,7 +1615,7 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
 
   value += textcodec->toUnicode( buffer.begin(), bit - buffer.begin() );
 
-  // kDebug(5320) << "value now: \"" << value << "\"";
+  // kDebug() << "value now: \"" << value << "\"";
   // cleanup:
   delete dec;
 }
@@ -1564,10 +1624,12 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
 //  - permutes rfc2231 continuations when the total number of parts
 //    exceeds 10 (other-sections then becomes *xy, ie. two digits)
 
-bool parseParameterList( const char* &scursor, const char * const send,
-                         QMap<QString,QString> &result, bool isCRLF )
+bool parseParameterListWithCharset( const char* &scursor,
+                                                const char * const send,
+                                                QMap<QString,QString> &result,
+                                                QByteArray& charset, bool isCRLF )
 {
-  // parse the list into raw attribute-value pairs:
+// parse the list into raw attribute-value pairs:
   QMap<QString,QStringOrQPair> rawParameterList;
   if (!parseRawParameterList( scursor, send, rawParameterList, isCRLF ) ) {
     return false;
@@ -1586,9 +1648,15 @@ bool parseParameterList( const char* &scursor, const char * const send,
   QTextCodec * textcodec = 0;
   QString attribute;
   QString value;
-  enum Modes {
+  enum Mode {
     NoMode = 0x0, Continued = 0x1, Encoded = 0x2
-  } mode;
+  };
+
+  enum EncodingMode {
+    NoEncoding,
+    RFC2047,
+    RFC2231
+  };
 
   QMap<QString,QStringOrQPair>::Iterator it, end = rawParameterList.end();
 
@@ -1605,24 +1673,37 @@ bool parseParameterList( const char* &scursor, const char * const send,
       // and extract the information from the new raw attribute:
       value.clear();
       attribute = it.key();
-      mode = NoMode;
-      // is the value encoded?
+      int mode = NoMode;
+      EncodingMode encodingMode = NoEncoding;
+
+      // is the value rfc2331-encoded?
       if ( attribute.endsWith( asterisk ) ) {
         attribute.truncate( attribute.length() - 1 );
-        mode = (Modes) ((int) mode | Encoded);
+        mode |= Encoded;
+        encodingMode = RFC2231;
+      }
+      // is the value rfc2047-encoded?
+      if( !(*it).qstring.isNull() && (*it).qstring.contains( "=?" ) ) {
+        mode |= Encoded;
+        encodingMode = RFC2047;
       }
       // is the value continued?
       if ( attribute.endsWith( asteriskZero ) ) {
         attribute.truncate( attribute.length() - 2 );
-        mode = (Modes) ((int) mode | Continued);
+        mode |= Continued;
       }
       //
       // decode if necessary:
       //
       if ( mode & Encoded ) {
-        decodeRFC2231Value( rfc2231Codec, textcodec,
-                            false, /* isn't continuation */
-                            value, (*it).qpair );
+        if ( encodingMode == RFC2231 ) {
+          decodeRFC2231Value( rfc2231Codec, textcodec,
+                              false, /* isn't continuation */
+                              value, (*it).qpair, charset );
+        }
+        else if ( encodingMode == RFC2047 ) {
+          value += decodeRFC2047String( (*it).qstring.toLatin1(), charset );
+        }
       } else {
         // not encoded.
         if ( (*it).qpair.first ) {
@@ -1652,7 +1733,7 @@ bool parseParameterList( const char* &scursor, const char * const send,
         // encoded
         decodeRFC2231Value( rfc2231Codec, textcodec,
                             true, /* is continuation */
-                            value, (*it).qpair );
+                            value, (*it).qpair, charset );
       } else {
         // not encoded
         if ( (*it).qpair.first ) {
@@ -1672,6 +1753,14 @@ bool parseParameterList( const char* &scursor, const char * const send,
   return true;
 }
 
+
+bool parseParameterList( const char* &scursor, const char * const send,
+                         QMap<QString,QString> &result, bool isCRLF )
+{
+  QByteArray charset;
+  return parseParameterListWithCharset( scursor, send, result, charset, isCRLF );
+}
+
 static const char * const stdDayNames[] = {
   "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
@@ -1687,7 +1776,7 @@ static bool parseDayName( const char* &scursor, const char * const send )
   for ( int i = 0 ; i < stdDayNamesLen ; ++i ) {
     if ( qstrnicmp( scursor, stdDayNames[i], 3 ) == 0 ) {
       scursor += 3;
-      // kDebug(5320) << "found" << stdDayNames[i];
+      // kDebug() << "found" << stdDayNames[i];
       return true;
     }
   }
@@ -2037,6 +2126,58 @@ bool parseDateTime( const char* &scursor, const char * const send,
   return true;
 }
 
+Headers::Base *extractFirstHeader( QByteArray &head )
+{
+  int endOfFieldBody = 0;
+  bool folded = false;
+  Headers::Base *header = 0;
+
+  int startOfFieldBody = head.indexOf( ':' );
+  const int endOfFieldHeader = startOfFieldBody;
+
+  if ( startOfFieldBody > -1 ) {    //there is another header
+    startOfFieldBody++; //skip the ':'
+    if ( head[startOfFieldBody] == ' ' ) { // skip the space after the ':', if there
+      startOfFieldBody++;
+    }
+    endOfFieldBody = findHeaderLineEnd( head, startOfFieldBody, &folded );
+
+    QByteArray rawType = head.left( endOfFieldHeader );
+    QByteArray rawFieldBody = head.mid( startOfFieldBody, endOfFieldBody - startOfFieldBody );
+    if ( folded ) {
+      rawFieldBody = unfoldHeader( rawFieldBody );
+    }
+    // We might get an invalid mail without a field name, don't crash on that.
+    if ( !rawType.isEmpty() ) {
+      header = HeaderFactory::self()->createHeader( rawType );
+    }
+    if( !header ) {
+      //kWarning() << "Returning Generic header of type" << rawType;
+      header = new Headers::Generic( rawType );
+    }
+    header->from7BitString( rawFieldBody );
+
+    head.remove( 0, endOfFieldBody + 1 );
+  } else {
+    head.clear();
+  }
+
+  return header;
+}
+
+Headers::Base::List parseHeaders( const QByteArray &head )
+{
+  Headers::Base::List ret;
+  Headers::Base *h;
+
+  QByteArray copy = head;
+  while( ( h = extractFirstHeader( copy ) ) ) {
+    ret << h;
+  }
+
+  return ret;
+}
+
 } // namespace HeaderParsing
 
 } // namespace KMime
